"""
Simple, non-threaded, run command.  Stores command in sharedstate for internal
inspection in bucket "__internals.run".  Can use this for cache.  Value of
state bucket key is:  (occurred_time, status, output, output_error)
"""

import subprocess
import logging
import time

from log import log

import sharedstate


def Run(command, env=None):
  """Actually run the command on the local machine.  Blocks until complete.
  
  Args:
    command: string, command to run in shell
    env: dict, values to set as environment variables for this command
  """
  
  output_error = '' #Later, how to handle reading the timing stream between the two?  It's lost...

  #log('Run: %s' % command)

  # Subprocess is beautiful and finally makes this a pleasant experience!
  #   Imagine, OUTPUT, ERRORS and EXIT CODE!!!  Not exclusively choosing two!
  #   Newbs: rejoice in your ignorance.
  pipe = subprocess.Popen(command, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE, shell=True, env=env)
  status = pipe.wait()
  output = pipe.stdout.read()
  output_error = pipe.stderr.read()
  
  # Close the pipes
  pipe.stderr.close()
  pipe.stdout.close()

  # Report any non-0 exit code...
  if status != 0:
    log('Non-Zero Exit Code: %s: %s: %s' % (status, command, output_error), logging.INFO)

  # Cache in shared state for internal viewing
  #TODO(g): Come up with a better way to restrict logging these.  rrdtool is a hog...
  if not command.startswith('rrdtool'):
    sharedstate.Set('__internals.run', command, (time.time(), status, output, output_error))

  return (status, output, output_error)
  

def OutputParsePrepare(output):
  """Any script that needs parsing may be helped by having all tabs converted to
  spaces and all duplicate spaces converted to tabs.
  
  Returns: list of strings, the parse-ready output, returns as a list of lines
  
  TODO(g): Respect quotes, as an option.  Right now all quoted strings will
      still be compressed, and optionally, this might be something we dont want.
  """
  output = output.replace('\t', ' ')
  while '  ' in output:
    output = output.replace('  ', ' ')
  
  lines = output.split('\n')
  
  return lines


def MakePathSafeToCreate(text, replace_char='_'):
  """Converts any unsafe characters in the text to path safe characters.
  
  Returns: string, safe for saving as a path name
  """
  #TODO(g): This is ghetto, replace
  replace_needed = '-+=\\|/[]{}()!@#$%^&*();:`~'
  
  # Cycle through all the chars in the string, and replace them
  for char in replace_needed:
    while char in text:
      text = text.replace(char, replace_char)
  
  return text
